% "Copyright (c) 2008 Stanford University. All rights reserved.
% 
% Permission to use, copy, modify, and distribute this software and
% its documentation for any purpose, without fee, and without written
% agreement is hereby granted, provided that the above copyright
% notice, the following two paragraphs and the author appear in all
% copies of this software.
% 
% IN NO EVENT SHALL STANFORD UNIVERSITY BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF STANFORD UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
% 
% STANFORD UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
% PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND STANFORD UNIVERSITY
% HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS."
% 
%  
% This function applies the derived RBF fit to test samples, and obtain the
% error measures
% 
% @author HyungJune Lee, Martin Wicke, Branislav Kusy, Leonidas Guibas
%         (hyungjune.lee@gmail.com)
% 
% @date   Nov 7 2008
%
% usage:
%
% eval_error(data_type, prev_k, train_burst, test_burst, average_tr, koborov, numcenters_or_subsampling, rbfcoeff, centers, test_set)
%
% returns:
%   no return value
%
% parameters:
%   data_type: 2=RSSI, 3=PRR, 4=LQI
%   prev_k: length of history
%   train_burst: the number of samples per burst used in training(***)
%   test_burst: the number of samples per burst used in testing
%   average_tr: average samples before fitting(***)
%   korobov: 0=use subsampling of paths for centers; 1=use korobov sampling(***)
%   numcenters_or_subsampling:  subsample every x'th sample (if koborov==0)(***)
%   numcenters_or_subsampling:  number of centers to use (if koborov==1)
%   rbfcoeff: rbf coefficients derived by prepare()
%   centers: rbf center points derived by prepare()
%   test_set: test set vectors consisting of 1=exp1, 7=exp7, 8=exp8, 9=exp9
%             [1 8]: exp1, exp8 for test sets
%
%   *** means (just used for file naming rule)

function [pos_errors, path_errors] = eval_error(data_type, prev_k, train_burst, test_burst, average_tr, koborov, numcenters_or_subsampling, rbfcoeff, centers, test_set)

details = false;
do_figures = false;

global data_path;
global result_path;

pos_errors = [];
path_errors = [];

geo = [];
melt_map;

dt = 0.6666;
MIN_RSSI = -95;
MIN_LQI = 50;
nodes = 11;

average = 1;
burst_size = test_burst;

for tsi=1:length(test_set)
    test_file = [];
    time_file = [];
    landmark_file = [];
    
    switch (test_set(tsi))
        case 1
            test_file = [test_file;
                 'm_exp1_02.txt'; 'm_exp1_03.txt'; 'm_exp1_04.txt'; 'm_exp1_05.txt';
                 'm_exp1_06.txt'; 'm_exp1_07.txt'; 'm_exp1_08.txt'; 'm_exp1_09.txt'; 'm_exp1_10.txt';
                 'm_exp1_11.txt'];
             time_file = [time_file;
                 'c_exp1_02.txt'; 'c_exp1_03.txt'; 'c_exp1_04.txt'; 'c_exp1_05.txt';
                 'c_exp1_06.txt'; 'c_exp1_07.txt'; 'c_exp1_08.txt'; 'c_exp1_09.txt'; 'c_exp1_10.txt';
                 'c_exp1_11.txt'];
             landmark_file = [landmark_file;
                 's_exp1.txt'; 's_exp1.txt'; 's_exp1.txt'; 's_exp1.txt';
                 's_exp1.txt'; 's_exp1.txt'; 's_exp1.txt'; 's_exp1.txt'; 's_exp1.txt';
                 's_exp1.txt'];

        case 7
            test_file = [test_file;
                 'm_exp7_01.txt'; 'm_exp7_02.txt'; 'm_exp7_03.txt'; 'm_exp7_04.txt'; 'm_exp7_05.txt'];
            time_file = [time_file;
                 'c_exp7_01.txt'; 'c_exp7_02.txt'; 'c_exp7_03.txt'; 'c_exp7_04.txt'; 'c_exp7_05.txt'];
            landmark_file = [landmark_file;
                 's_exp7.txt'; 's_exp7.txt'; 's_exp7.txt'; 's_exp7.txt'; 's_exp7.txt']; 

        case 8
            test_file = [test_file;
                 'm_exp8_01.txt'; 'm_exp8_02.txt'; 'm_exp8_03.txt'; 'm_exp8_04.txt'; 'm_exp8_05.txt'];
            time_file = [time_file;
                 'c_exp8_01.txt'; 'c_exp8_02.txt'; 'c_exp8_03.txt'; 'c_exp8_04.txt'; 'c_exp8_05.txt'];
            landmark_file = [landmark_file;
                 's_exp8.txt'; 's_exp8.txt'; 's_exp8.txt'; 's_exp8.txt'; 's_exp8.txt']; 
    %         test_file = [test_file;
    %              'm_exp8_04.txt'];
    %         time_file = [time_file;
    %              'c_exp8_04.txt'];
    %         landmark_file = [landmark_file;
    %              's_exp8.txt']; 
        case 9
            test_file = [test_file;
                 'm_exp9_01.txt'; 'm_exp9_02.txt'; 'm_exp9_03.txt'; 'm_exp9_04.txt'; 'm_exp9_05.txt'];
            time_file = [time_file;
                 'c_exp9_01.txt'; 'c_exp9_02.txt'; 'c_exp9_03.txt'; 'c_exp9_04.txt'; 'c_exp9_05.txt'];
            landmark_file = [landmark_file;
                 's_exp9.txt'; 's_exp9.txt'; 's_exp9.txt'; 's_exp9.txt'; 's_exp9.txt'];      
    end    
    
	test_file = horzcat(repmat(data_path, length(test_file(:,1)), 1), test_file);
	time_file = horzcat(repmat(data_path, length(time_file(:,1)), 1), time_file);
	landmark_file = horzcat(repmat(data_path, length(landmark_file(:,1)), 1), landmark_file);

    ds_size = length(test_file(:,1));
	Xt_rbf = [];
	Yt_rbf = [];

	tv_error = [];
	tv_error_kf = [];
    geom_error = [];
    
    v_test_num = [];

	for ds=1:ds_size
        clear fdata time;
        disp(['Reading data ' test_file(ds,:)]);

        fdata = load(test_file(ds,:));
        fdata = fdata(:,[1 2 4 5]);
        [m n] = size(fdata);

        %disp(['  reading times ' time_file(ds,:)]);

        time = load(time_file(ds,:));
        time = time - time(1);
        time = round(time/1000/dt)+1;
        time = time(2:end);

        %disp(['  reading landmarks ' landmark_file(ds,:)]);

        landmark = load(landmark_file(ds,:));

        %disp(['  preprocessing...']);

        for i=1:m
            if (i==1)
                tmp_index = fdata(i,1);
                index = [tmp_index i];
            elseif (fdata(i,1) ~= tmp_index)
                    tmp_index = fdata(i,1);
                    index = [index ; tmp_index i];
            end
            if (fdata(i,3) >= 128)
                fdata(i,3) = fdata(i,3) - 256 - 45;
            else
                fdata(i,3) = fdata(i,3) - 45;
            end
        end
        max_x = max(fdata(:,2));

        max_sec = floor(max_x/30)+1;

        if (time(end) > max_sec)
            time(end) = max_sec;
        end

        tmp_count = 0;
        len = nodes*max_sec;
        rdata = [];
        disp_max_x = ceil(dt*max_sec) + 3;

        for i=1:nodes
            rdata = [rdata; i*ones(max_sec,1) zeros(max_sec, 3)];
        end

        for i=1:m
            if (mod(fdata(i,2),30) >= burst_size)
                continue;
            end
            pt = (fdata(i,1)-1)*max_sec + (floor(fdata(i,2)/30)+1);
            if average
                sub_seq = 1;
            else
                sub_seq = mod(fdata(i,2),30)+1;
            end
            rdata(pt,1,sub_seq) = fdata(i,1);
            rdata(pt,2,sub_seq) = rdata(pt,2,sub_seq) + fdata(i,3);
            rdata(pt,3,sub_seq) = rdata(pt,3,sub_seq) + 1;
            rdata(pt,4,sub_seq) = rdata(pt,4,sub_seq) + fdata(i,4);
        end

        tmp_prr = zeros(len,1);

        for i=1:len
            if average
                j=1;
                if (rdata(i,3,j) == 0)
                    rdata(i,2,j) = MIN_RSSI;
                    rdata(i,4,j) = MIN_LQI;
                else
                    rdata(i,2,j) = rdata(i,2,j)/rdata(i,3,j);
                    rdata(i,4,j) = rdata(i,4,j)/rdata(i,3,j);
                    rdata(i,3,j) = rdata(i,3,j)/burst_size;
                end
            else
                for j=1:burst_size
                    if (rdata(i,3,j) == 0)
                        rdata(i,2,j) = MIN_RSSI;
                        rdata(i,4,j) = MIN_LQI;
                    end
                end
            end
        end

        clear data tmpRSSI;
        for i=1:nodes
            if average
                data(i,:,1) = [rdata((i-1)*max_sec+1:i*max_sec,data_type,1)'];
                tmpRSSI(i,:,1) = [rdata((i-1)*max_sec+1:i*max_sec,2,1)'];
            else
                for j=1:burst_size
                    data(i,:,j) = [rdata((i-1)*max_sec+1:i*max_sec,data_type,j)'];
                    tmpRSSI(i,:,j) = [rdata((i-1)*max_sec+1:i*max_sec,2,j)'];
                end
            end
        end

        Yt_rbf = [];

        discard_sample = [];
        for i=1:max_sec
            if (i < prev_k)
                Yt_rbf = [Yt_rbf zeros(2,1)];
                continue;
            end
            Xt_rbf = [];
            discard_flag= 0;
            if (tmpRSSI(:,i,1) == MIN_RSSI*ones(nodes,1))
                discard_sample = [discard_sample i];
                discard_flag = 1;
            else
                for j=1:prev_k-1
                    if (length(find(discard_sample == i-j)) ~= 0)
                        discard_flag = 1;
                        break;
                    end
                end
            end
            tmpY = zeros(2,1);

            if (discard_flag ==1)
                Yt_rbf = [Yt_rbf tmpY];
            else
                for j=1:nodes
                    Xt_rbf = [Xt_rbf; data(j, i-prev_k+1:i,1)'];
                end
                for j=1:2
                    tmpY(j) = rbfinterp_k(Xt_rbf, centers, rbfcoeff(j));
                end
                Yt_rbf = [Yt_rbf tmpY];
            end
        end

        count=1;
        real_traj = [];
        tmp_error = [];
        tmp_error_kf = [];

        t_geom_error = [];

        test_vector = [];
        if (landmark(2) == 2)
            init_vel = 1;
        else
            init_vel = -1;
        end
        % disable kalman filter
%         [Yt_rbf_kf] = melt08_kf(Yt_rbf, geo(1,2:3)', init_vel);
        Yt_rbf_kf = Yt_rbf;
        Yt_rbf_kf = Yt_rbf_kf(1:2,:);

        for i=time(1):max_sec
            if (i < prev_k)
                if (i==time(1))
                    ref = geo(landmark(count),2:3)';
                    count = count + 1;
                end
                continue;
            end
            if (count > length(time))
                break;
            end
            discard_flag = 0;
            for j=0:prev_k-1
                if (length(find(discard_sample == i-j)) ~= 0)
                    discard_flag = 1;
                    break;
                end
            end
            if (discard_flag == 1)
                if (i==time(count))
                    ref = geo(landmark(count),2:3)';
                    count = count + 1;
                end
                continue;
            end

            if (i == time(count)) 
                ref = geo(landmark(count),2:3)';
                count = count + 1;
            else
                next_loc = geo(landmark(count),2:3)';
                prev_loc = geo(landmark(count-1),2:3)';
                ref = (i-time(count-1))*(next_loc-prev_loc)/(time(count)-time(count-1))+prev_loc;
            end  
            tv_error = [tv_error norm(Yt_rbf(:, i) - ref)];
            tmp_error = [tmp_error norm(Yt_rbf(:, i) - ref)];

            tv_error_kf = [tv_error_kf norm(Yt_rbf_kf(:, i) - ref)];         
            tmp_error_kf = [tmp_error_kf norm(Yt_rbf_kf(:, i) - ref)];

            real_traj = [real_traj ref];
            test_vector = [test_vector i];
        end 
        for i=1:length(test_vector)
            j = test_vector(i);
            [val idx] = min(sqrt((Yt_rbf(1,j)-real_traj(1,:)).^2 + (Yt_rbf(2,j)-real_traj(2,:)).^2));
            geom_error = [geom_error val];

            t_geom_error = [t_geom_error val];
        end
        v_test_num = [v_test_num length(test_vector)];

        error_q = quantile(tmp_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0]);       
        error_q_kf = quantile(tmp_error_kf, [0.0 0.025 0.25 0.50 0.75 0.975 1.0]);
        geo_error_q = quantile(t_geom_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0]);
        switch (data_type)
            case 2
                dtype = 'rssi';
            case 3
                dtype = 'prr';
            case 4
                dtype = 'lqi';
        end
         
        eval_fname = ['exp',num2str(test_set(tsi)),'-',num2str(ds),'-',dtype,'-hs',num2str(prev_k),'-b',num2str(train_burst),'-',num2str(test_burst),...
        '-avg',num2str(average_tr),'-k',num2str(koborov),'-s',num2str(numcenters_or_subsampling)];
        fid = fopen([result_path,eval_fname,'.txt'],'w');
        fprintf(fid,'===== %s Result =====\n', eval_fname);

        fprintf(fid,'\n= Position Error =\n');
        fprintf(fid,'1)Mean\n');
        fprintf(fid,'%0.4f\n\n', mean(tmp_error));
        fprintf(fid, '2)Quantile [min, 0.025 0.25 0.50 0.75 0.975 max]\n');
        fprintf(fid, '%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n', error_q(1), error_q(2),...
            error_q(3), error_q(4), error_q(5), error_q(6), error_q(7));

%         fprintf(fid,'\n= Position Error with Kalman Filtering =\n');
%         fprintf(fid,'1)Mean\n');
%         fprintf(fid,'%0.4f\n\n', mean(tmp_error_kf));
%         fprintf(fid, '2)Quantile [min, 0.025 0.25 0.50 0.75 0.975 max]\n');
%         fprintf(fid, '%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n', error_q_kf(1), error_q_kf(2),...
%             error_q_kf(3), error_q_kf(4), error_q_kf(5), error_q_kf(6), error_q_kf(7));

        fprintf(fid,'\n= Path Error =\n');
        fprintf(fid,'1)Mean\n\n');
        fprintf(fid,'%0.4f\n', mean(t_geom_error));
        fprintf(fid, '2)Quantile [min, 0.025 0.25 0.50 0.75 0.975 max]\n');
        fprintf(fid, '%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n', geo_error_q(1), geo_error_q(2),...
            geo_error_q(3), geo_error_q(4), geo_error_q(5), geo_error_q(6), geo_error_q(7));

        fclose(fid);

        if details
            disp(['===== ',eval_fname, ' Result =====']);
            disp('= Position error =');
            mean(tmp_error)
            quantile(tmp_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0])

%         disp('= Position error with Kalman Filtering =');
%         mean(tmp_error_kf)
%         quantile(tmp_error_kf, [0.0 0.025 0.25 0.50 0.75 0.975 1.0])

            disp('= Path error =');
            mean(t_geom_error)
            quantile(t_geom_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0])
        end
       
        time_vector = [];
        for i=1:length(time)
            j = time(i);
            if (length(find(test_vector == j)) == 0 | j < prev_k)
                continue;
            end
            time_vector = [time_vector j];
        end

        if do_figures
            figure;
            subplot(2,1,1);
            set(gca, 'LineWidth', 2, 'FontSize', 15, 'FontWeight', 'bold');
            plot(geo(:,2), geo(:,3), 'rx', 'LineWidth', 3);
            hold on;
            % plot(Ytr_rbf(1,:), Ytr_rbf(2,:),'o');
            plot_dir(Yt_rbf(1,test_vector)', Yt_rbf(2,test_vector)');
            plot(Yt_rbf(1,time_vector), Yt_rbf(2,time_vector), '*', 'LineWidth', 3);

            xlabel('X (m)');
            ylabel('Y (m)');
            title(['Trajectory for ',eval_fname]);

            subplot(2,1,2);
            set(gca, 'LineWidth', 2, 'FontSize', 15, 'FontWeight', 'bold');
            plot(geo(:,2), geo(:,3), 'rx', 'LineWidth', 3);
            hold on;
            plot(real_traj(1,:), real_traj(2,:),'r.');
            plot(Yt_rbf(1,test_vector), Yt_rbf(2,test_vector),'b.');
            for i=1:length(real_traj(1,:))
                j = test_vector(i);
                if (time(1) < prev_k)
                    vpx = (real_traj(1,i)-Yt_rbf(1,j));
                    vpy = (real_traj(2,i)-Yt_rbf(2,j));
                    quiver(Yt_rbf(1,j), Yt_rbf(2,j), vpx, vpy, 0, 'm');
                else
                    vpx = (real_traj(1,i)-Yt_rbf(1,j));
                    vpy = (real_traj(2,i)-Yt_rbf(2,j));
                    quiver(Yt_rbf(1,j), Yt_rbf(2,j), vpx, vpy, 0, 'm');
                end
            end
            grid on;
            plot(Yt_rbf(1,time_vector), Yt_rbf(2,time_vector), '*', 'LineWidth', 3);

            xlabel('X (m)');
            ylabel('Y (m)');
            title(['Error Graph for ', eval_fname]);

            orient portrait;
            print('-dpng', [resultpath eval_fname '.png']);
            close;

%         figure;
%         subplot(2,1,1);
%         set(gca, 'LineWidth', 2, 'FontSize', 15, 'FontWeight', 'bold');
%         plot(geo(:,2), geo(:,3), 'rx', 'LineWidth', 3);
%         hold on;
%         % plot(Ytr_rbf(1,:), Ytr_rbf(2,:),'o');
%         plot_dir(Yt_rbf_kf(1,test_vector)', Yt_rbf_kf(2,test_vector)');
%         plot(Yt_rbf_kf(1,time_vector), Yt_rbf_kf(2,time_vector), '*', 'LineWidth', 3);
% 
%         xlabel('X (m)');
%         ylabel('Y (m)');
% 
%         title(['Trajectory after KF for ',eval_fname]);
% 
% 
%         subplot(2,1,2);
%         set(gca, 'LineWidth', 2, 'FontSize', 15, 'FontWeight', 'bold');
%         plot(geo(:,2), geo(:,3), 'rx', 'LineWidth', 3);
%         hold on;
%         plot(real_traj(1,:), real_traj(2,:),'r.');
%         plot(Yt_rbf_kf(1,test_vector), Yt_rbf_kf(2,test_vector),'b.');
%         for i=1:length(real_traj(1,:))
%             j = test_vector(i);
%             if (time(1) < prev_k)
%                 vpx = (real_traj(1,i)-Yt_rbf_kf(1,j));
%                 vpy = (real_traj(2,i)-Yt_rbf_kf(2,j));
%                 quiver(Yt_rbf_kf(1,j), Yt_rbf_kf(2,j), vpx, vpy, 0, 'm');
%             else
%                 vpx = (real_traj(1,i)-Yt_rbf_kf(1,j));
%                 vpy = (real_traj(2,i)-Yt_rbf_kf(2,j));
%                 quiver(Yt_rbf_kf(1,j), Yt_rbf_kf(2,j), vpx, vpy, 0, 'm');
%             end
%         end
%         grid on;
%         plot(Yt_rbf_kf(1,time_vector), Yt_rbf_kf(2,time_vector), '*', 'LineWidth', 3);
% 
%         xlabel('X (m)');
%         ylabel('Y (m)');
%         title(['Error Graph after KF for ', eval_fname]);
% 
%         orient portrait;
%         print('-dpng', [eval_fname,'-kf']);
%         close;
        end
    end
    
    disp('===== Overall Result =====');

    Num_Tests = sum(v_test_num)

    eval_fname1 = ['pos-error-exp',num2str(test_set(tsi)),'-',dtype,'-hs',num2str(prev_k),'-b',num2str(train_burst),'-',num2str(test_burst),...
            '-avg',num2str(average_tr),'-k',num2str(koborov),'-s',num2str(numcenters_or_subsampling)];
%     eval_fname2 = ['pos-kf-error-exp',num2str(test_set(tsi)),'-',dtype,'-hs',num2str(prev_k),'-b',num2str(train_burst),'-',num2str(test_burst),...
%             '-avg',num2str(average),'-k',num2str(koborov),'-s',num2str(numcenters_or_subsampling)];
    eval_fname3 = ['path-error-exp',num2str(test_set(tsi)),'-',dtype,'-hs',num2str(prev_k),'-b',num2str(train_burst),'-',num2str(test_burst),...
            '-avg',num2str(average_tr),'-k',num2str(koborov),'-s',num2str(numcenters_or_subsampling)];

    disp('= Position Error =');
    mean(tv_error)
    quantile(tv_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0])

%     disp('= Position Error with Kalman Filtering =');
%     mean(tv_error_kf)
%     quantile(tv_error_kf, [0.0 0.025 0.25 0.50 0.75 0.975 1.0])

    disp('= Path Error =');
    mean(geom_error)
    quantile(geom_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0])

    tv_error_q = quantile(tv_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0]);
    tv_error_q_kf = quantile(tv_error_kf, [0.0 0.025 0.25 0.50 0.75 0.975 1.0]);
    gv_error_q = quantile(geom_error, [0.0 0.025 0.25 0.50 0.75 0.975 1.0]);

    tv_error = tv_error';
    tv_error_kf = tv_error_kf';
    geom_error = geom_error';
    
    save([result_path,eval_fname1,'.txt'],'tv_error','-ASCII');
%     save([eval_fname2,'.txt'],'tv_error_kf','-ASCII');
    save([result_path,eval_fname3,'.txt'],'geom_error','-ASCII');

    eval_fname_f = ['exp',num2str(test_set(tsi)),'-',dtype,'-hs',num2str(prev_k),'-b',num2str(train_burst),'-',num2str(test_burst),...
            '-avg',num2str(average_tr),'-k',num2str(koborov),'-s',num2str(numcenters_or_subsampling)];

    fid = fopen([result_path,eval_fname_f,'.txt'],'w');
    fprintf(fid,'===== %s Overall Result =====\n', eval_fname_f);

    fprintf(fid,'\n= Position Error =\n');
    fprintf(fid,'1)Mean\n');
    fprintf(fid,'%0.4f\n\n', mean(tv_error));
    fprintf(fid, '2)Quantile [min, 0.025 0.25 0.50 0.75 0.975 max]\n');
    fprintf(fid, '%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n', tv_error_q(1), tv_error_q(2),...
        tv_error_q(3), tv_error_q(4), tv_error_q(5), tv_error_q(6), tv_error_q(7));

%     fprintf(fid,'\n= Position Error with Kalman Filtering =\n');
%     fprintf(fid,'1)Mean\n');
%     fprintf(fid,'%0.4f\n\n', mean(tv_error_kf));
%     fprintf(fid, '2)Quantile [min, 0.025 0.25 0.50 0.75 0.975 max]\n');
%     fprintf(fid, '%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f\n', tv_error_q_kf(1), tv_error_q_kf(2),...
%         tv_error_q_kf(3), tv_error_q_kf(4), tv_error_q_kf(5), tv_error_q_kf(6), tv_error_q_kf(7));

    fprintf(fid,'\n= Path Error =\n');
    fprintf(fid,'1)Mean\n');
    fprintf(fid,'%0.4f\n\n', mean(geom_error));
    fprintf(fid, '2)Quantile [min, 0.025 0.25 0.50 0.75 0.975 max]\n');
    fprintf(fid, '%0.4f %0.4f %0.4f %0.4f %0.4f %0.4f %0.4f  [RSSI]\n', gv_error_q(1), gv_error_q(2),...
        gv_error_q(3), gv_error_q(4), gv_error_q(5), gv_error_q(6), gv_error_q(7));


    fprintf(fid, '\n\nTOTAL # of TEST SAMPLES = %d\n\n', sum(v_test_num));
    fclose(fid);

    % concat all for return value    
    pos_errors = [pos_errors tv_error'];
    path_errors = [path_errors geom_error'];
end

pos_fname = ['pos-error-',dtype,'-hs',num2str(prev_k),'-b',num2str(train_burst),'-',num2str(test_burst),...
            '-avg',num2str(average_tr),'-k',num2str(koborov),'-s',num2str(numcenters_or_subsampling)];
tmp_pos_errors = pos_errors';
save([result_path,pos_fname,'.txt'],'tmp_pos_errors','-ASCII');

path_fname = ['path-error-',dtype,'-hs',num2str(prev_k),'-b',num2str(train_burst),'-',num2str(test_burst),...
            '-avg',num2str(average_tr),'-k',num2str(koborov),'-s',num2str(numcenters_or_subsampling)];
tmp_path_errors = path_errors';
save([result_path,path_fname,'.txt'],'tmp_path_errors','-ASCII');

end
