function [f] = rbfinterp(x, c, options)
% 
% Alex Chirokov, alex.chirokov@gmail.com
% 16 Feb 2006
% 
% Modified by HyungJune Lee, Martin Wicke, Branislav Kusy, Leonidas Guibas 
%             (hyungjune.lee@gmail.com)

tic;
nodes     = c;
phi       = options.('rbfphi');
rbfconst  = options.('RBFConstant');
rbfcoeff  = (options.('rbfcoeff'))';

[dim              n] = size(nodes);
[dimPoints  nPoints] = size(x);

if (dim~=dimPoints)
  error(sprintf('x should have the same number of rows as an array used to create RBF interpolation'));
end;

f = zeros(1, nPoints);
r = zeros(1, n);

for i=1:1:nPoints
    r = (x(:,i)*ones(1,n)) - nodes;
    r = sqrt(sum(r.*r, 1));

    % constant part
    s = rbfcoeff(n+1); 
    % s = 0;
    
    % RBF part
    s = s + sum(rbfcoeff(1:n).*feval(phi, r, rbfconst));

    % linear part
    for k=1:dim
        s=s+rbfcoeff(k+n+1)*x(k,i);     % linear part
    end
    
    f(i) = s;
end;

if (strcmp(options.('Stats'),'on'))
    fprintf('Interpolation at %d points was computed in %e sec\n', length(f), toc);    
end;
