% "Copyright (c) 2008 Stanford University. All rights reserved.
% 
% Permission to use, copy, modify, and distribute this software and
% its documentation for any purpose, without fee, and without written
% agreement is hereby granted, provided that the above copyright
% notice, the following two paragraphs and the author appear in all
% copies of this software.
% 
% IN NO EVENT SHALL STANFORD UNIVERSITY BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
% ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
% IF STANFORD UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
% DAMAGE.
% 
% STANFORD UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES,
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
% MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
% PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND STANFORD UNIVERSITY
% HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS."
% 
%
% Given a matrix whose columns contain constraint points, 
% return a matrix whose columns contain RBF center positions
% 
% @author HyungJune Lee, Martin Wicke, Branislav Kusy, Leonidas Guibas 
%         (hyungjune.lee@gmail.com)
% 
% @date   Nov 7 2008
 

function centers = select_centers(ratio, constraints)

if ratio > 1
    error('More centers than constraints...');
end

[dim,n] = size(constraints);

% compute final number of centers
m = floor(n * ratio);

selector = zeros(1, m);
count = 0;
acc = ratio;

for i = 1:n
    if acc - 1 >= 0
        count = count + 1;
        selector(count) = i;
        acc = acc - 1;
    end

    acc = acc + ratio;
    
    %disp(['i=' int2str(i) ', count=' int2str(count) ', acc=' num2str(acc)]);
end

selector = selector(selector > 0);

disp(['selected ' num2str(length(selector)) ' centers. Checking for duplicates...']);

centers = constraints(:,selector);

[dim, m] = size(centers);

keep = 1:m;

for i = 1:m
    if find(keep == i)
        for j = keep(keep>i)        
            if norm(centers(:,j) - centers(:,i)) < 1
                keep = keep(keep~=j);
            end
        end
    end
end

centers = centers(:,keep);

disp(['selected ' num2str(length(keep)) ' centers.']);
